@@ -2,7 +2,7 @@ import itertools, re, sys
 
 from django.conf import urls
 from django.core import exceptions
-from django.db.models import base
+from django.db.models import base as models_base
 from django.db.models.sql import constants
 from django.utils import datastructures
 
@@ -15,6 +15,17 @@ import bson
 
 from tastypie_mongoengine import fields
 
+# When Tastypie accesses query terms used by QuerySet it assumes the interface of Django ORM. 
+# We use a mock Query object to provide the same interface and return query terms by MongoEngine. 
+# MongoEngine code might not expose these query terms, so we fallback to hard-coded values.
+
+QUERY_TERMS_ALL = getattr(queryset, 'QUERY_TERMS_ALL', ('ne', 'gt', 'gte', 'lt', 'lte', 'in', 'nin', 'mod', 'all', 'size', 'exists', 'not', 'within_distance', 'within_spherical_distance', 'within_box', 'within_polygon', 'near', 'near_sphere','contains', 'icontains', 'startswith', 'istartswith', 'endswith', 'iendswith', 'exact', 'iexact', 'match'))
+
+class Query(object):
+    query_terms = dict([(query_term, None) for query_term in QUERY_TERMS_ALL])
+
+queryset.QuerySet.query = Query()
+
 CONTENT_TYPE_RE = re.compile('.*; type=([\w\d-]+);?')
 
 class NOT_HYDRATED:
@@ -158,8 +169,13 @@ class MongoEngineModelDeclarativeMetaclass(resources.ModelDeclarativeMetaclass):
 
         for field_name in field_names:
             if field_name == 'resource_uri':
-                # Delete resource_uri from fields if this is mongoengine.EmbeddedDocument
-                if meta and issubclass(meta.object_class, mongoengine.EmbeddedDocument):
+                if hasattr(new_class, '_parent'):
+                    if new_class._parent._meta.object_class and issubclass(new_class._parent._meta.object_class, mongoengine.EmbeddedDocument):
+                        # TODO: We do not support yet nested resources
+                        # If parent is embedded document, then also this one do not have its own resource_uri
+                        del(new_class.base_fields[field_name])
+                elif new_class._meta.object_class and issubclass(new_class._meta.object_class, mongoengine.EmbeddedDocument):
+                    # Embedded documents which are not in lists (do not have _parent) do not have their own resource_uri
                     del(new_class.base_fields[field_name])
             if field_name in new_class.declared_fields:
                 continue
@@ -254,7 +270,7 @@ class MongoEngineResource(resources.ModelResource):
 
     def _wrap_polymorphic(self, resource, fun):
         object_class = self._meta.object_class
-        queryset = self._meta.queryset
+        qs = self._meta.queryset
         base_fields = self.base_fields
         fields = self.fields
         try:
@@ -268,7 +284,7 @@ class MongoEngineResource(resources.ModelResource):
             return fun()
         finally:
             self._meta.object_class = object_class
-            self._meta.queryset = queryset
+            self._meta.queryset = qs
             self.base_fields = base_fields
             self.fields = fields
 
@@ -383,7 +399,7 @@ class MongoEngineResource(resources.ModelResource):
                 # Tastypie also ignores missing fields in PUT,
                 # so we check for missing field here
                 # (https://github.com/toastdriven/django-tastypie/issues/496)
-                if not bundle.data.has_key(field_object.instance_name):
+                if field_object.instance_name not in bundle.data:
                     if field_object._default is not tastypie_fields.NOT_PROVIDED:
                         if callable(field_object.default):
                             value = field_object.default()
@@ -465,19 +481,19 @@ class MongoEngineResource(resources.ModelResource):
         try:
             return super(MongoEngineResource, self).obj_get(request, **kwargs)
         except self._meta.object_class.DoesNotExist, e:
-            exp = base.subclass_exception('DoesNotExist', (self._meta.object_class.DoesNotExist, exceptions.ObjectDoesNotExist), self._meta.object_class.DoesNotExist.__module__)
+            exp = models_base.subclass_exception('DoesNotExist', (self._meta.object_class.DoesNotExist, exceptions.ObjectDoesNotExist), self._meta.object_class.DoesNotExist.__module__)
             raise exp(*e.args)
         except queryset.DoesNotExist, e:
-            exp = base.subclass_exception('DoesNotExist', (queryset.DoesNotExist, exceptions.ObjectDoesNotExist), queryset.DoesNotExist.__module__)
+            exp = models_base.subclass_exception('DoesNotExist', (queryset.DoesNotExist, exceptions.ObjectDoesNotExist), queryset.DoesNotExist.__module__)
             raise exp(*e.args)
         except self._meta.object_class.MultipleObjectsReturned, e:
-            exp = base.subclass_exception('MultipleObjectsReturned', (self._meta.object_class.MultipleObjectsReturned, exceptions.MultipleObjectsReturned), self._meta.object_class.MultipleObjectsReturned.__module__)
+            exp = models_base.subclass_exception('MultipleObjectsReturned', (self._meta.object_class.MultipleObjectsReturned, exceptions.MultipleObjectsReturned), self._meta.object_class.MultipleObjectsReturned.__module__)
             raise exp(*e.args)
         except queryset.MultipleObjectsReturned, e:
-            exp = base.subclass_exception('MultipleObjectsReturned', (queryset.MultipleObjectsReturned, exceptions.MultipleObjectsReturned), queryset.MultipleObjectsReturned.__module__)
+            exp = models_base.subclass_exception('MultipleObjectsReturned', (queryset.MultipleObjectsReturned, exceptions.MultipleObjectsReturned), queryset.MultipleObjectsReturned.__module__)
             raise exp(*e.args)
         except mongoengine.ValidationError, e:
-            exp = base.subclass_exception('DoesNotExist', (queryset.DoesNotExist, exceptions.ObjectDoesNotExist), queryset.DoesNotExist.__module__)
+            exp = models_base.subclass_exception('DoesNotExist', (queryset.DoesNotExist, exceptions.ObjectDoesNotExist), queryset.DoesNotExist.__module__)
             raise exp(*e.args)
 
     def obj_update(self, bundle, request=None, **kwargs):
@@ -704,7 +720,7 @@ class MongoEngineListResource(MongoEngineResource):
             try:
                 obj = self.obj_get(request, **kwargs)
             except (queryset.DoesNotExist, exceptions.ObjectDoesNotExist):
-                raise NotFound("A document instance matching the provided arguments could not be found.")
+                raise exceptions.NotFound("A document instance matching the provided arguments could not be found.")
 
         getattr(self.instance, self.attribute).pop(int(obj.pk))
         self.instance.save()
@@ -722,6 +738,9 @@ class MongoEngineListResource(MongoEngineResource):
         }
 
         if hasattr(obj, 'parent'):
+            # pk could not exist in the case of nested resources, but we should not come here in this
+            # case as we should remove resource_uri from fields in MongoEngineModelDeclarativeMetaclass
+            # TODO: Support nested resources
             kwargs['pk'] = obj.parent.pk
         else:
             kwargs['pk'] = self.instance.pk
